import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.Border;

/**
 * Panel in which drawing occurs.
 * 
 * @author CS2334. Modified by: ?????
 * @version 20161103
 */

public class DrawPanel extends JPanel
{
    /**
     * Makes Eclipse happy
     */
    private static final long serialVersionUID = 1L;
    /** The list of shapes to be drawn. */
    private ArrayList<Shape> shapeList = new ArrayList<Shape>();
    /** A temporary shape that is also drawn */
    private Shape tempShape;
    /** Index of chosen shape being edited */
    private int shapeIndex;

    /** Coordinates for button-down event. */
    private int x0;
    private int y0;
    /** Coordinates for button-up event. */
    private int x1;
    private int y1;
    /** Indicate if a shape is currently being drawn. */
    private boolean drawingFlag;

    /** Reference to the frame in which the panel is a member. */
    private DrawFrame frame;

    //////////////////////////////////////////////////////////////
    /**
     * Class defines the behavior inside of the drawing panel in response to
     * mouse events
     * 
     */
    public class MouseHandler extends MouseAdapter
    {
        /**
         * Respond to a button-down event.
         * 
         * @param e Mouse event
         */
        @Override
        public void mousePressed(MouseEvent e)
        {
            // Remember this starting coordinate
            // TODO

            // are we in edit mode?
            if (frame.isEditing())
            {
                // create point where the mouse was clicked
                // TODO

                // find which shape was clicked
                // loop through shapes in stack fashion, LIFO
                // TODO
               
                // if the shape contains the point, set the shapeIndex
                // to be the index in the shapeList
                // TODO

                // find if the shape is filled
                // set fillBox to match the status of the shape
                // TODO

                // get color of the shape
                // set the color of the frame to match the shape's color
                // TODO

                       
                        
                    
                
            }
            else if (frame.isDeleting()) // are we in delete mode?
            {
                // create point where the mouse was clicked
                // TODO

                // find which shape was clicked
                // loop through shapes in stack fashion, LIFO
                
                // If the shape contains the point, prompt the user for
                // a confirmation to delete
                // TODO
        
                // Check answer
                // If yes, remove the shape
                // TODO
                    
                
            }
            else // we're drawing a shape
            {
                // Indicate that drawing of a shape has begun
                
            }
        }

        /**
         * Respond to a button-up event.
         * 
         * @param e Mouse event
         */
        @Override
        public void mouseReleased(MouseEvent e)
        {
            
            // Are we currently drawing?
            if (drawingFlag)
            {
                // Yes - we are currently drawing

                // Coordinates of the cursor
                // TODO

                // Indicate that we are no longer drawing
                // TODO

                // We no longer need a temporary shape
                // TODO

                // Create the shape given the current state
                // TODO

                // Add the shape to the panel list if the shape exists
                // TODO
                
                //repaint
                repaint();
            }
        }

        /**
         * Address a mouse movement event (movement only when the button is
         * down)
         * 
         * @param e Mouse event
         */
        @Override
        public void mouseDragged(MouseEvent e)
        {
            // TODO just comments
            // Are we currently drawing a shape?
            if (drawingFlag)
            {
                // Yes
                // Note the current coordinates
                // TODO

                // Create a temporary shape
                // TODO

                // repaint
                repaint();
            }
        }

        /**
         * Given the current state of the control panel and the selected points
         * in the draw panel, create a new shape.
         * 
         * @return The newly created shape
         */
        private Shape createShape()
        {
            // Identify the upper left and lower right coordinates
            int xUL = Math.min(x0, x1);
            int yUL = Math.min(y0, y1);
            int xLR = Math.max(x0, x1);
            int yLR = Math.max(y0, y1);

            // Create a new object, depending on what is selected
            if (frame.isOval())
            {
                // create and return an Oval
                // TODO
            }
            else if (frame.isRectangle())
            {
                // create and return a Rectangle
                // TODO
            }
            else if (frame.isTriangle())
            {
                // create and return a Triangle
                // TODO
            }
            else if (frame.isDiamond())
            {
                // 4 quadrants-ish; axis is y=|x| and y = -|x|
                // the coordinate frame is rotated 45 degrees

                // get the change in x and y directions
                int deltaX = x1 - x0;
                int deltaY = y1 - y0;

                // initialize a Point (has to have x,y values, set to zero here
                // but not significant - could be any numbers)
                Point start = new Point(0, 0);

                // initialize edge length to be zero, again, not significant -
                // just a placeholder
                int edgeLength = 0;

                if (deltaX >= 0 && Math.abs(deltaX) >= Math.abs(deltaY))
                {
                    // right quadrant

                    // start point is where the initial click was
                    start = new Point(x0, y0);
                    // calculate edge length
                    edgeLength = (int) ((x1 - x0) * Math.sqrt(2) / 2);
                }
                else if (deltaX < 0 && Math.abs(deltaX) >= Math.abs(deltaY))
                {
                    // left quadrant

                    // start point - same y, but x is where the mouse is
                    // released or at when being dragged
                    start = new Point(x1, y0);
                    // calculate edge length
                    edgeLength = (int) ((x0 - x1) * Math.sqrt(2) / 2);
                }
                else if (deltaY >= 0 && Math.abs(deltaY) >= Math.abs(deltaX))
                {
                    // bottom quadrant
                    
                    // start point - x is initial x - 1/2 * change in y
                    // y is average of y values
                    start = new Point(x0 - (y1 - y0) / 2, (y1 + y0) / 2);
                    // calculate edge length
                    edgeLength = (int) ((y1 - y0) * Math.sqrt(2) / 2);
                }
                else if (deltaY < 0 && Math.abs(deltaY) >= Math.abs(deltaX))
                {
                    // top quadrant
                    
                    // start point - x is initial x - 1/2 * change in y
                    // y is average of y values
                    start = new Point(x0 - (y0 - y1) / 2, (y1 + y0) / 2);
                    //calculate edge length
                    edgeLength = (int) ((y0 - y1) * Math.sqrt(2) / 2);
                }
                // create diamond
                Shape s = new Diamond(start, edgeLength, frame.getColor(),
                                frame.isFilled());
                // return shape
                return s;
            }
            // Should not get here, but be safe
            return null;
        }
    }

    //////////////////////////////////////////////////////////////
    /**
     * Constructor: Create the draw panel
     * 
     * @param frame The Frame in which the panel is embedded
     */
    public DrawPanel(DrawFrame frame)
    {
        this.frame = frame;

        // Border for the frame
        Border blackline = BorderFactory.createLineBorder(Color.black);
        this.setBorder(blackline);

        // Initially, we are currently not editing a shape
        drawingFlag = false;

        // Set the panel to be focusable: we will receive mouse events
        this.setFocusable(true);

        // Create a handler for the mouse events
        MouseHandler listener = new MouseHandler();

        // Listen for button events
        this.addMouseListener(listener);

        // Also listen for mouse movement events
        this.addMouseMotionListener(listener);
    }

    /**
     * This method adds shapes to the set of shapes.
     * 
     * @param shape The shape to be added to the list of shapes.
     */
    public void addShape(Shape shape)
    {
        shapeList.add(shape);
    }

    /**
     * This method draws the shapes in the set of shapes.
     * 
     * @param g A Graphics object
     */
    protected void paintComponent(Graphics g)
    {
        // TODO Draw all the shapes including the temporary shape.
    }

    /**
     * Return the list of shapes
     * 
     * @return The list of shapes
     */
    protected ArrayList<Shape> getShapeList()
    {
        return shapeList;
    }

    /**
     * Set the list of shapes to a new set
     * 
     * @param shapeList to copy into our panel
     */
    protected void setShapeList(ArrayList<Shape> shapeList)
    {
        if (shapeList != null)
        {
            this.shapeList = shapeList;
            repaint();
        }
    }

    /**
     * Remove all shapes from the list.
     */
    protected void clearShapeList()
    {
        shapeList.clear();
        repaint();
    }

    /**
     * Removed the selected shape from the shape list.
     * 
     * @param index The index of the selected shape.
     */
    protected void removeShape(int index)
    {
        shapeList.remove(index);
        repaint();
    }

    /**
     * Getter for shape index.
     * 
     * @return Index of the shape chosen by the user.
     */
    public int getShapeIndex()
    {
        return shapeIndex;
    }

}
